#include "../unity/unity.h"

#include "./ivec.h"
#include "./ivec_ext.h"

#include <stdlib.h>

void setUp(void) {}
void tearDown(void) {}

bool div_by_three(int elem) {
  return elem % 3 == 0;
}

void test_ivec_filter(void) {
  IntVec* xs = ivec_new();

  int ys[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
  for (size_t i = 0; i < 11; ++i) {
    ivec_push(xs, ys[i]);
  }

  ivec_filter(xs, div_by_three);

  TEST_ASSERT_EQUAL_INT(4, ivec_length(xs));
  TEST_ASSERT_EQUAL(0, *ivec_at(xs, 0));
  TEST_ASSERT_EQUAL(3, *ivec_at(xs, 1));
  TEST_ASSERT_EQUAL(6, *ivec_at(xs, 2));
  TEST_ASSERT_EQUAL(9, *ivec_at(xs, 3));

  ivec_free(xs);
}

void test_ivec_sum(void) {
  IntVec* xs = ivec_new();
  for (int i = 0; i < 5; ++i) {
    ivec_push(xs, i);
  }

  TEST_ASSERT_EQUAL(10, ivec_sum(xs));

  ivec_free(xs);
}

void test_ivec_product(void) {
  IntVec* xs = ivec_new();
  for (int i = 1; i < 5; ++i) {
    ivec_push(xs, i);
  }

  TEST_ASSERT_EQUAL(24, ivec_product(xs));

  ivec_free(xs);
}

int main(void) {
  UNITY_BEGIN();
  RUN_TEST(test_ivec_sum);
  RUN_TEST(test_ivec_product);
  RUN_TEST(test_ivec_filter);
  return UNITY_END();
}
